"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@typescript-eslint/utils");
const createRule_1 = __importDefault(require("../util/createRule"));
const memoizeComponent = (0, createRule_1.default)({
    name: 'memoize-component',
    meta: {
        docs: {
            description: 'React components should be memoized',
            recommended: 'warn',
        },
        messages: {
            general: 'Memoize this component',
        },
        type: 'problem',
        schema: [
            {
                type: 'object',
                properties: {},
                additionalProperties: false,
            },
        ],
        fixable: 'code',
    },
    defaultOptions: [{ ignoreHooks: [] }],
    create(context) {
        return {
            VariableDeclaration(node) {
                var _a;
                const declaration = node.declarations[0];
                if (declaration.type === utils_1.AST_NODE_TYPES.VariableDeclarator) {
                    const init = declaration.init;
                    if (!init) {
                        // Not a component
                        return;
                    }
                    const name = declaration.id.type === utils_1.AST_NODE_TYPES.Identifier
                        ? declaration.id.name
                        : undefined;
                    const defaultExportNode = (_a = node.parent.body) === null || _a === void 0 ? void 0 : _a.find(item => {
                        return (item.type ===
                            utils_1.AST_NODE_TYPES.ExportDefaultDeclaration);
                    });
                    if (init.type === utils_1.AST_NODE_TYPES.ArrowFunctionExpression &&
                        name &&
                        // Shallow checking for component declaration
                        name.charAt(0) === name.charAt(0).toUpperCase()) {
                        if (!defaultExportNode) {
                            // TODO: Only report if component
                            context.report({
                                node,
                                messageId: 'general',
                                *fix(fixer) {
                                    yield fixer.insertTextBefore(init, 'memo(');
                                    yield fixer.insertTextAfter(init, ')');
                                },
                            });
                            return;
                        }
                        const isDefaultExportMemoized = (() => {
                            const exportDeclaration = defaultExportNode.declaration;
                            if (exportDeclaration &&
                                exportDeclaration.type ===
                                    utils_1.AST_NODE_TYPES.CallExpression) {
                                const callee = exportDeclaration.callee;
                                if (callee.type === utils_1.AST_NODE_TYPES.Identifier &&
                                    callee.name === 'memo') {
                                    return true;
                                }
                            }
                        })();
                        if (isDefaultExportMemoized) {
                            return;
                        }
                        context.report({
                            node: defaultExportNode,
                            messageId: 'general',
                            *fix(fixer) {
                                yield fixer.insertTextBefore(defaultExportNode.declaration, 'memo(');
                                yield fixer.insertTextAfter(defaultExportNode.declaration, ')');
                            },
                        });
                        return;
                    }
                    if (init.type === utils_1.AST_NODE_TYPES.CallExpression) {
                        const callee = init.callee;
                        if (callee.type === utils_1.AST_NODE_TYPES.Identifier) {
                            if (callee.name !== 'memo') {
                                context.report({
                                    node,
                                    messageId: 'general',
                                    *fix(fixer) {
                                        yield fixer.insertTextBefore(init, 'memo(');
                                        yield fixer.insertTextAfter(init, ')');
                                    },
                                });
                                return;
                            }
                        }
                    }
                }
            },
        };
    },
});
exports.default = memoizeComponent;
